from CyXTraX.simulation import CylindricalProjection, load_mesh
from CyXTraX.data import scones_mesh
from CyXTraX.common import MeshObject

import numpy as np
from matplotlib import pyplot as plt
from pathlib import Path


FOLDER = Path(__file__)
TEMP_FOLDER = FOLDER.parent.parent / 'temp'


def main():
    # Create the cylindrical projection interface
    cyl_proj = CylindricalProjection()


    # Load the test object 'olaf'. It is automatically installed to your site-packages
    print(f'STL Path: {scones_mesh.object_path}')

    # Obect are loaded in a dataclass: MeshObject.
    print(f'Mesh is a MeshObject: {isinstance(scones_mesh, MeshObject)}')
    scones_mesh.position_mm = np.array([0, 0, 0])
    scones_mesh.orientation_quat = np.array([0., -0., 0., 0.1])
    
    
    # The `load_mesh` function load a list of MeshObject
    load_mesh(cyl_proj, [scones_mesh])

    # Set local position of cylindrical projection
    map_position = np.array([0, 0, 0])

    # Generate local cylindrical projection
    half_rays = cyl_proj.compute_projection(map_position, output_full_ray_projection=False)
    # output_full_ray_projection=True is the same As 
    # half_rays = cyl_proj.compute_projection(position, output_full_ray_projection=False)
    # full_rays = cyl_proj.convert_rays(half_rays)
    full_rays = cyl_proj.compute_projection(map_position, output_full_ray_projection=True)
    

    # Make some nice plots!
    fig = plt.figure(figsize=(12, 6))

    ax1 = fig.add_subplot(121)
    ax1.imshow(half_rays)
    ax1.set_xlabel(r'$Z$ / mm')
    ax1.set_ylabel(r'$\alpha$ / rad')
    ax1.set_title('Half Rays')
    ax1.set_xticks([0, 1000, 2000]) 
    ax1.set_xticklabels([r'$-3141.5$', r'$0.$',  r'$3141.5$']) 
    ax1.set_yticks([0, 1000, 2000]) 
    ax1.set_yticklabels([r'$-\pi$', r'$0.$',  r'$\pi$']) 

    ax2 = fig.add_subplot(122)
    ax2.imshow(full_rays)
    ax2.set_xlabel(r'$Z$ / mm')
    ax2.set_ylabel(r'$\alpha$ / rad')
    ax2.set_title('Full Rays')
    ax2.set_xticks([0, 1000, 2000]) 
    ax2.set_xticklabels([r'$-3141.5$', r'$0.$',  r'$3141.5$']) 
    ax2.set_yticks([0, 1000, 2000]) 
    ax2.set_yticklabels([r'$-\pi$', r'$0.$',  r'$\pi$']) 

    plt.savefig(TEMP_FOLDER / 'scones.png')


if __name__ == '__main__':
    main()